from django.contrib.contenttypes.models import ContentType

from extras.plugins import PluginTemplateExtension
from netbox_storage.models import LogicalVolume, StorageConfigurationDrive, \
    TemplateConfigurationDrive, Drive, Partition, LinuxDevice, MountedVolume


class RelatedDrives(PluginTemplateExtension):
    model = "virtualization.virtualmachine"

    def left_page(self):
        obj = self.context.get("object")

        lv = LogicalVolume.objects.all()

        storage_configuration = StorageConfigurationDrive.objects.filter(virtual_machine=obj)

        platform = obj.platform
        if platform is not None:
            if platform.name.lower().__contains__('windows'):
                return self.render(
                    "netbox_storage/inc/windowsvolume_box.html",
                    extra_context={
                        # "volumes": volumes,
                        # "unmapped_drives": unmapped_drives,
                        "type": type(obj.platform),
                    },
                )
            elif platform.name.lower().__contains__('linux'):
                return self.render(
                    "netbox_storage/inc/linuxvolume_box.html"
                )
        else:
            return self.render(
                "netbox_storage/inc/unknown_os_box.html",
                extra_context={
                    "lv": lv,
                    "storage_configuration": storage_configuration
                }
            )


class TemplateVolumeConfig(PluginTemplateExtension):
    model = "dcim.platform"

    def right_page(self):
        obj = self.context.get("object")

        drives_id = TemplateConfigurationDrive.objects.values('drive').filter(platform=obj)
        partition_dict = {
        }
        for drive_id in drives_id:
            print(f"Drive ID: {drive_id['drive']}")
            drive = Drive.objects.get(pk=drive_id['drive'])

            drive_type_id = ContentType.objects.get(app_label='netbox_storage', model='drive').pk
            linux_device_type_id = ContentType.objects.get(app_label='netbox_storage', model='linuxdevice').pk
            # Get Linux Device of Drive e.g. /dev/sda
            linux_device_drive = LinuxDevice.objects.get(content_type_id=drive_type_id,
                                                         object_id=drive_id['drive'],
                                                         type='Disk')

            if LinuxDevice.objects.filter(content_type_id=linux_device_type_id,
                                          object_id=linux_device_drive.pk,
                                          type='Partition').count() == 0:
                partition_dict[drive] = None
            else:
                partition_dict[drive] = list(LinuxDevice.objects.filter(content_type_id=linux_device_type_id,
                                                                        object_id=linux_device_drive.pk,
                                                                        type='Partition'))

        return self.render(
            "netbox_storage/inc/template_drive_box.html",
            extra_context={
                "partition_dict": partition_dict
            }
        )

    def left_page(self):
        mounted_volumes = MountedVolume.objects.all()
        return self.render(
            "netbox_storage/inc/template_volume_box.html",
            extra_context={
                "mounted_volumes": mounted_volumes
            }
        )

    def full_width_page(self):
        obj = self.context.get("object")

        drives = TemplateConfigurationDrive.objects.values('drive').filter(platform=obj)
        drives_id = []
        lonely_drive = []
        linux_devices = []
        for drive_id in drives:
            drives_id.append(drive_id['drive'])
            # if LinuxDevice.objects.filter(linux_device_drives=drive_id['drive']).count() == 0:
            drive = Drive.objects.get(pk=drive_id['drive'])
            lonely_drive.append(drive)
#                print(LinuxDevice.objects.filter(linux_device_drives=drive_id['drive']))

        partitions = Partition.objects.filter(drive_id__in=drives_id)

        return self.render(
            "netbox_storage/inc/template_volume_box.html",
            extra_context={
                "lonely_drives": lonely_drive,
                "partitions": partitions,
                "linux_devices": linux_devices
            }
        )


template_extensions = [RelatedDrives, TemplateVolumeConfig]



from netbox.views import generic

from netbox_storage.filters import MountedVolumeFilter
from netbox_storage.forms import (
    MountedVolumeImportForm,
    MountedVolumeFilterForm,
    MountedVolumeForm,
    MountedVolumeBulkEditForm
)

from netbox_storage.models import MountedVolume
from netbox_storage.tables import MountedVolumeTable


class MountedVolumeListView(generic.ObjectListView):
    queryset = MountedVolume.objects.all()
    filterset = MountedVolumeFilter
    filterset_form = MountedVolumeFilterForm
    table = MountedVolumeTable


class MountedVolumeView(generic.ObjectView):
    queryset = MountedVolume.objects.all()


class MountedVolumeEditView(generic.ObjectEditView):

    queryset = MountedVolume.objects.all()
    form = MountedVolumeForm
    default_return_url = "plugins:netbox_storage:mountedvolume_list"


class MountedVolumeDeleteView(generic.ObjectDeleteView):
    queryset = MountedVolume.objects.all()
    default_return_url = "plugins:netbox_storage:mountedvolume_list"


class MountedVolumeBulkImportView(generic.BulkImportView):
    queryset = MountedVolume.objects.all()
    model_form = MountedVolumeImportForm
    table = MountedVolumeTable
    default_return_url = "plugins:netbox_storage:mountedvolume_list"


class MountedVolumeBulkEditView(generic.BulkEditView):
    queryset = MountedVolume.objects.all()
    filterset = MountedVolumeFilter
    table = MountedVolumeTable
    form = MountedVolumeBulkEditForm


class MountedVolumeBulkDeleteView(generic.BulkDeleteView):
    queryset = MountedVolume.objects.all()
    table = MountedVolumeTable



from extras.plugins import PluginMenuButton, PluginMenuItem
from extras.plugins import PluginMenu
from utilities.choices import ButtonColorChoices

drive_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:drive_list",
    link_text="Drives",
    permissions=["netbox_storage.drive_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:drive_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_drive"],
        ),
        PluginMenuButton(
            "plugins:netbox_storage:drive_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_storage.add_drive"],
        ),
    ),
)

lv_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:logicalvolume_list",
    link_text="Logical Volumes",
    permissions=["netbox_storage.storage_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:logicalvolume_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_storage"],
        ),
    ),
)
vg_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:volumegroup_list",
    link_text="Volume Groups",
    permissions=["netbox_storage.storage_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:volumegroup_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_storage"],
        ),
    ),
)
pv_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:physicalvolume_list",
    link_text="Physical Volume",
    permissions=["netbox_storage.storage_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:physicalvolume_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_storage"],
        ),
        PluginMenuButton(
            "plugins:netbox_storage:physicalvolume_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_storage.add_disk"],
        ),
    ),
)

filesystem_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:filesystem_list",
    link_text="Filesystem",
    permissions=["netbox_storage.storage_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:filesystem_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_storage"],
        ),
        PluginMenuButton(
            "plugins:netbox_storage:filesystem_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_storage.add_storage"],
        ),
    ),
)


partition_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:partition_list",
    link_text="Partition",
    permissions=["netbox_storage.disk_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:partition_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_disk"],
        ),
        PluginMenuButton(
            "plugins:netbox_storage:partition_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_storage.add_disk"],
        ),
    ),
)

linux_device_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:linuxdevice_list",
    link_text="Linux Device",
    permissions=["netbox_storage.disk_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:linuxdevice_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_disk"],
        ),
        PluginMenuButton(
            "plugins:netbox_storage:linuxdevice_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_storage.add_disk"],
        ),
    ),
)

mounted_volume_menu_item = PluginMenuItem(
    link="plugins:netbox_storage:mountedvolume_list",
    link_text="Mounted Volume",
    permissions=["netbox_storage.disk_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_storage:mountedvolume_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_storage.add_disk"],
        ),
        PluginMenuButton(
            "plugins:netbox_storage:mountedvolume_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_storage.add_disk"],
        ),
    ),
)

menu = PluginMenu(
    label="NetBox Storage",
    groups=(
        (
            "General Configuration",
            (
                filesystem_menu_item,
            ),
        ),
        (
            "Linux & LVM Configuration",
            (
                mounted_volume_menu_item,
                lv_menu_item,
                vg_menu_item,
                pv_menu_item,
                linux_device_menu_item
            ),
        ),
        (
            "Windows Volume Configuration",
            (
                partition_menu_item,
            ),
        ),
        (
            "Storage Configuration",
            (
                drive_menu_item,
            ),
        ),

    ),
    icon_class="mdi mdi-disc",
)
